# -*- coding: utf-8 -*-

import datetime
import json
import urllib
import time

import xbmc
import xbmcvfs

import xbmcup.app
import xbmcup.gui
import xbmcup.system

from files import Files


COVER_MYMAGIVTV = xbmcup.system.fs('home://addons/plugin.video.mymagictv/resources/media/icons/mymagictv.png')
COVER_BACKWARD = xbmcup.system.fs('home://addons/plugin.video.mymagictv/resources/media/icons/backward.png')
COVER_FORWARD = xbmcup.system.fs('home://addons/plugin.video.mymagictv/resources/media/icons/forward.png')
COVER_FAVORITE = xbmcup.system.fs('home://addons/plugin.video.mymagictv/resources/media/icons/favorite.png')


class Index(xbmcup.app.Handler):
    noauth = None

    def handle(self):
        if self._auth():
            try_num, groups = 20, None
            while try_num and groups is None:
                if try_num < 20:
                    xbmc.sleep(1000)
                try_num -= 1
                groups = Files(xbmc.translatePath('special://temp/mymagictv/m3u')).read('groups.json')
                if not groups and xbmcvfs.exists('special://temp/mymagictv/noauth.pid'):
                    xbmcvfs.delete('special://temp/mymagictv/noauth.pid')
                    xbmcup.gui.alert(xbmcup.app.lang[50004], title=u'myMAGicTV')
                    break
            else:
                if groups:
                    for group in json.loads(groups):
                        # TODO: Remove checking to length
                        name = group[1]
                        if len(group) > 2:
                            name += ' - ' + str(group[2])
                        protected = group[3] if len(group) > 3 else 0
                        self.item(name, self.link('channels', gid=group[0], protected=protected), folder=True, cover=COVER_MYMAGIVTV)
        self.render(mode='list', content='files')

    def _auth(self):
        try_num, token = 3, None
        while try_num:
            if not xbmcvfs.exists('special://temp/mymagictv/noauth.pid'):
                return True
            f = xbmcvfs.File('special://temp/mymagictv/noauth.pid')
            temp = f.read()
            f.close()
            if temp == token:
                xbmc.sleep(1000)
            else:
                try_num -= 1
                token = temp
                old_login = xbmcup.app.setting['login']
                old_password = xbmcup.app.setting['password']
                while True:
                    login = xbmcup.gui.prompt(xbmcup.app.lang[50001], old_login)
                    if not login:
                        return False
                    password = xbmcup.gui.password(xbmcup.app.lang[50002], old_password)
                    if not password:
                        return False
                    if old_login != login or old_password != password:
                        break
                xbmcup.app.setting['login'] = login
                xbmcup.app.setting['password'] = password
        return False


class Channels(xbmcup.app.Handler):
    def handle(self):
        if self.argv['protected']:
            code = xbmcup.app.setting['code']
            if code:
                rcode = xbmcup.gui.password(xbmcup.app.lang[50501])
                if code != rcode:
                    self.render(mode='thumb')
                    return

        channels = Files(xbmc.translatePath('special://temp/mymagictv/m3u')).read('%s.json' % self.argv['gid'])
        if channels:
            xmltv = Files(xbmc.translatePath('special://temp/mymagictv/xmltv'))
            for channel in json.loads(channels):
                name = channel['name']
                menu = None
                epg = 'channel.%s.json' % (channel['cid'] if channel.get('cid', '').isdigit() else urllib.quote_plus(name.lower().encode('utf8')).replace(' ', '_'))
                if channel['arc'] and xmltv.read(epg):
                    menu = [(xbmcup.app.lang[50401], self.resolve(
                        'archive',
                        name=channel['name'],
                        url=channel['url'],
                        arc=channel['arc'],
                        epg=epg
                    ))]
                    name = u'[COLOR FF00FF00]%s[/COLOR]' % name
                self.item(name, self.play(channel['url']), cover=channel['logo'], menu=menu)
        self.render(mode='thumb')


class Archive(xbmcup.app.Handler):
    def handle(self):
        epg = Files(xbmc.translatePath('special://temp/mymagictv/xmltv')).read(self.argv['epg'])
        if epg:
            current = int(time.time())
            now = int(time.strftime('%Y%m%d%H%M%S', time.gmtime(current)))
            arc = int(time.strftime('%Y%m%d%H%M%S', time.gmtime(current - 60*60*self.argv['arc'])))
            select = []
            for programme in json.loads(epg):
                if now >= programme[0] > arc:
                    start = int(time.mktime(time.strptime(str(programme[0]), '%Y%m%d%H%M%S'))) - time.altzone
                    select.append((start, u'[COLOR FF00FF00]%s[/COLOR] %s' % (time.strftime('%H:%M', time.localtime(start)), programme[1])))
            if select:
                start = xbmcup.gui.select(self.argv['name'], select)
                if start:
                    return self.argv['url'] + '?time_start=%s' % (start + int(xbmcup.app.setting['timeshift']))


def run():
    plugin = xbmcup.app.Plugin()
    plugin.route(None, Index)
    plugin.route('channels', Channels)
    plugin.route('archive', Archive)
    plugin.run()
